package attic;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Paint;
import java.awt.Stroke;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;


import org.apache.commons.collections15.Transformer;

import costo.kml2java.framework.ExecutableLTS;
import costo.kml2java.framework.IService;
import costo.kml2java.framework.ProvidedService;
import costo.kml2java.framework.ServiceEvolutionListener;
import costo.kml2java.monitor.gui.basecomponents.MyLink;
import costo.kml2java.monitor.gui.basecomponents.MyNode;
import edu.uci.ics.jung.algorithms.layout.ISOMLayout;
import edu.uci.ics.jung.algorithms.layout.Layout;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.SparseMultigraph;
import edu.uci.ics.jung.graph.util.EdgeType;
import edu.uci.ics.jung.visualization.BasicVisualizationServer;
import edu.uci.ics.jung.visualization.decorators.ToStringLabeller;
import edu.uci.ics.jung.visualization.renderers.Renderer.VertexLabel.Position;

public class CopyOfSimpleTest implements ServiceEvolutionListener {

	private final Map<String, MyNode> NODES = new HashMap<String, MyNode>();
	private final Map<String, MyLink> LINKS = new HashMap<String, MyLink>();
	private Graph<MyNode, MyLink> graph;
	private Layout<MyNode, MyLink> layout;
	private BasicVisualizationServer<MyNode, MyLink> visuserver;
	private String ltsfile;
	private JTextArea ta;
	private JFrame frame;
	private JPanel graphArea;
	private Map<ProvidedService, Graph<MyNode, MyLink>> graphs = new HashMap<ProvidedService, Graph<MyNode, MyLink>>();

	public CopyOfSimpleTest(String ltsfile) {
		super();
		this.ltsfile = ltsfile;
		try {
			graph = createGraphFromFile(ltsfile);
			// mytests();
		} catch (IOException e) {
			// 
			e.printStackTrace();
		}

	}

	public CopyOfSimpleTest(ExecutableLTS lts) {
		graph = new SparseMultigraph<MyNode, MyLink>();
		createGraphFromLTS(lts);
		// mytests();
	}

	public void createGraphFromLTS(ExecutableLTS lts) {
		Graph<MyNode, MyLink> graph = new SparseMultigraph<MyNode, MyLink>();
		for (String s : lts.getOutGoing().keySet()) {
			MyNode n = new MyNode(s);
			graph.addVertex(n);
			NODES.put(s, n);

		}
		for (Map.Entry<String, String> tlab : lts.getTransitionLabels().entrySet()) {
			String[] tab = tlab.getKey().split("___");
			MyLink ln = new MyLink(tlab.getKey());
			ln.setLabel(tlab.getValue());
			graph.addEdge(ln, NODES.get(tab[0]), NODES.get(tab[1]), EdgeType.DIRECTED);
			LINKS.put(tlab.getKey(), ln);
		}

	}

	public Graph<MyNode, MyLink> createGraphFromFile(String filepath) throws IOException {
		Graph<MyNode, MyLink> g = new SparseMultigraph<MyNode, MyLink>();
		BufferedReader read = new BufferedReader(new FileReader(filepath));
		String line = null;
		while ((line = read.readLine()) != null) {
			String[] tab = line.split(" ");
			if (tab[0].equals("state")) {
				MyNode n = new MyNode(tab[1]);
				g.addVertex(n);
				NODES.put(tab[1], n);
			} else if (tab[0].equals("transition")) {
				MyLink ln = new MyLink(tab[1]);
				g.addEdge(ln, NODES.get(tab[2]), NODES.get(tab[3]), EdgeType.DIRECTED);
				LINKS.put(tab[1], ln);
			}
		}
		return g;

	}

	/**
	 * @param args
	 * @throws IOException
	 */
	public static void main(String[] args) throws IOException {
		// AddNodeDemo.main(args);

	}

	/**
	 * @throws IOException
	 */
	public void display() {
		// graph = createGraphFromFile(ltsfile);
		layout = new ISOMLayout<MyNode, MyLink>(graph);
		layout.setSize(new Dimension(500, 500));
		visuserver = new BasicVisualizationServer<MyNode, MyLink>(layout);
		visuserver.setPreferredSize(new Dimension(550, 550));
		Transformer<MyNode, Paint> nodecolor = new Transformer<MyNode, Paint>() {

			@Override
			public Paint transform(MyNode arg0) {
				if (arg0.isSelected())
					return Color.RED;
				return Color.GREEN;

			}
		};

		Transformer<MyLink, Paint> edgecolor = new Transformer<MyLink, Paint>() {

			@Override
			public Paint transform(MyLink arg0) {
				if (arg0.isSelected())
					return Color.RED;
				return Color.GREEN;

			}
		};

		float dash[] = { 10.0f };
		final Stroke edgeStroke = new BasicStroke(1.0f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER, 10.0f, dash, 0.0f);
		Transformer<MyLink, Stroke> edgeStrokeTransformer = new Transformer<MyLink, Stroke>() {
			public Stroke transform(MyLink s) {

				return edgeStroke;

			}
		};
		visuserver.getRenderContext().setEdgeDrawPaintTransformer(edgecolor);
		// System.out.println();
		visuserver.getRenderContext().setVertexDrawPaintTransformer(nodecolor);
		visuserver.getRenderContext().setVertexFillPaintTransformer(nodecolor);
		visuserver.getRenderContext().setVertexLabelTransformer(new ToStringLabeller());
		visuserver.getRenderContext().setEdgeLabelTransformer(new ToStringLabeller());
		visuserver.getRenderer().getVertexLabelRenderer().setPosition(Position.CNTR);

		// JButton but = new JButton("waug");
		// final Random rand = new Random();
		// but.addActionListener(new ActionListener() {
		//
		// @Override
		// public void actionPerformed(ActionEvent e) {
		// // visuserver.getModel().setGraphLayout(new CircleLayout<MyNode,
		// // MyLink>(g));
		// visuserver.getModel().getRelaxer().pause();
		// for (MyNode node : NODES.values()) {
		// // node.setName(node.getName() + "Y");
		//
		// node.setSelected(rand.nextInt(4) == 0);
		//
		// }
		// for (MyLink ln : LINKS.values()) {
		// // node.setName(node.getName() + "Y");
		//
		// ln.setSelected(rand.nextInt(4) == 0);
		//
		// }
		// visuserver.getModel().getRelaxer().resume();
		// visuserver.repaint();
		// // Probleme avec le dolayout
		// }
		// });

		frame = new JFrame("Simple Graph View");
		frame.setPreferredSize(new Dimension(1600, 1200));
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.getContentPane().setLayout(new FlowLayout());
		frame.getContentPane().add(visuserver);
		// ta=new JTextArea();
		ta = new JTextArea("test", 15, 100);

		JScrollPane p = new JScrollPane(ta);
		frame.getContentPane().add(p);
		// frame.getContentPane().add(but);
		frame.pack();
		frame.setVisible(true);
	}

	@Override
	public void transitionFired(String tname, String currentState, IService serv, String log, Exception e) {
		// if (visuserver)
		// PlainDocument pa =(PlainDocument) ta.getDocument();
		// try {
		// pa.insertString(0, ta.getText()+ " "+log, null);
		// } catch (BadLocationException e) {
		// //
		// e.printStackTrace();
		// }
		// System.err.println(""+ta.getDocument()+ta.getText());
		ta.append("\n" + log);
		ta.revalidate();
		visuserver.getModel().getRelaxer().pause();
		for (MyNode node : NODES.values()) {
			// node.setName(node.getName() + "Y");

			node.setSelected(false);

		}

		NODES.get(currentState).setSelected(true);
		for (MyLink ln : LINKS.values()) {
			// node.setName(node.getName() + "Y");

			ln.setSelected(false);

		}
		MyLink selectedln = LINKS.get(tname);
		if (selectedln == null) {
			System.err.println("transition " + tname + " not found");
		} else {
			selectedln.setSelected(true);
		}
		visuserver.getModel().getRelaxer().resume();
		visuserver.repaint();
		frame.validate();
		// Probleme avec le dolayout

	}
}
